from datetime import datetime

from mcsniperpy.util.logs_manager import Color as color
from mcsniperpy.util.logs_manager import Logger as log
from mcsniperpy.util.request_manager import RequestManager


async def announce(
    username: str,
    authorization: str,
    session: RequestManager,
    full_url: str = "https://announcements-api.herokuapp.com/api/v1/announce",
) -> bool:
    headers = {"Authorization": authorization}
    json = {"name": username}
    resp, _, resp_json = await session.post(full_url, json=json, headers=headers)
    if resp.status < 300:
        log.info(f"{color.white}[{color.l_green}success{color.white}]{color.reset}")
        return True

    log.error(f"Failed to announce snipe | {resp.status} | {resp_json}")
    return False


DISCORD_RESP_CODES = {
    200: "ok",
    201: "created",
    204: "ok (no content)",
    304: "not modified",
    400: "bad request",
    401: "unauthorized",
    403: "forbidden",
    404: "not found",
    405: "method not allowed",
    429: "too many requests",
    502: "gateway unavailable",
    500: "server error",
}


async def gen_webhook_description(desc_format, name, session: RequestManager):
    resp, _, resp_json = await session.get(
        f"https://api.kqzz.me/api/namemc/searches/{name}"
    )

    searches: int = 0

    if resp.status < 300:
        searches = resp_json["searches"]
    else:
        searches = 0

    final_format = desc_format.format(
        name=name, searches=searches, namemcLink=f"https://namemc.com/profile/{name}"
    )

    return final_format


async def webhook_announce(
    webhook_url: str,
    session: RequestManager,
    embed_description: str,
    embed_title: str,
    prename: bool = False,
    embed_color: int = 0x779ABD,
):
    action = "prenamed" if prename else "sniped"
    embed_content = {
        "content": None,
        "embeds": [
            {
                "title": embed_title,
                "description": embed_description,
                "color": embed_color,
                "footer": {
                    "text": f"{action.title()} with MCsniperPY",
                    "icon_url": "https://mcsniperpy.com/img/old-logo.png",
                },
                "timestamp": datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%S.000Z"),
            }
        ],
    }
    resp, _, _ = await session.post(webhook_url, json=embed_content)
    if resp.status < 300:
        log.info(
            f"{color.white}[{color.l_green}success{color.white}]{color.reset} "
            "successfully sent discord webhook!"
        )
        return True

    log.error(
        f"failed to send Discord webhook | {resp.status} | "
        f"{DISCORD_RESP_CODES.get(resp.status, 'unknown')}"
    )



import configparser
import os
from pathlib import Path

from ..logs_manager import Logger as log
from ..utils import close

MCSNIPERPY_BACKEND_DIR = os.path.join(str(Path.home()), ".mcsniperpy")
MCSNIPERPY_CONFIG_FILE_PATH = os.path.join(
    MCSNIPERPY_BACKEND_DIR, "backend_config.ini")

DEFAULT_ACCOUNTS_FILE = """Clear this file and write accounts in this format
email:pass:answer:answer:answer

or

email:pass

you can separate those accounts by a new line if you would like to use multiple accounts.

# You can comment line out by making them start with a #"""


class BackConfig:
    """Backend configuration for MCsniperPY

    The main purpose of this is to store the path for the \"frontend\" directory.
    The frontend directory stores the config file and accounts file."""

    def __init__(self):

        self.config = configparser.ConfigParser()

        if not os.path.isdir(MCSNIPERPY_BACKEND_DIR):
            os.mkdir(MCSNIPERPY_BACKEND_DIR)

        if os.path.isfile(MCSNIPERPY_CONFIG_FILE_PATH):
            self.config.read(MCSNIPERPY_CONFIG_FILE_PATH)
        else:
            log.error("Failed to find MCsniperPY directory")
            log.info(
                "To fix this open your sniping folder (or make one) and type"
                " `mcsniperpy init` in it."
            )
            log.info(
                "Next, just re-run the sniper with the command you used to get here."
            )
            close(1)

        if not self.validate_sections():
            log.error(
                "invalid config file. please make sure it looks like:\n[sniper]\ninit_path ="
            )
            close(1)

    def validate_sections(self):
        necessary_sections = ["sniper"]
        sections = self.config.sections()
        log.debug(sections)
        return sections == necessary_sections


class Config:
    def __init__(self, config_path):
        log.debug(f"config path: {config_path}")
        user_config = configparser.ConfigParser(allow_no_value=True)
        user_config.read(config_path)

        self.config = user_config


def create_user_config() -> configparser.ConfigParser:
    user_config = configparser.ConfigParser(allow_no_value=True)
    user_config["sniper"] = {
        "timing_system": "kqzz_api",
        "auto_claim_namemc": "no",
        "snipe_requests": "3",
    }

    user_config["accounts"] = {
        "max_accounts": "30",
        "authentication_delay": "500",
        "start_authentication": "720",
    }

    user_config["skin"] = {
        "change_skin_on_snipe": "no",
        "skin_change_type": "url",
        "skin": "",
    }

    user_config["announce"] = {
        "announce_snipe": "no",
        "announcement_code": "",
        "webhook_urls": "",
        "webhook_format": "sniped `{name}` with `{searches}` searches!",
    }

    user_config.set(
        "skin",
        "; skin_change_type can be url, path, or username."
        "refer to docs for more info.",
    )

    return user_config


def populate_configs(no_confirm=False):
    if not no_confirm:
        if not log.yes_or_no(
            "Are you sure you want to initialize your config in your cwd:"
        ):
            close(1)

    config = configparser.ConfigParser()
    config["sniper"] = {"init_path": os.getcwd()}

    if not os.path.isdir(MCSNIPERPY_BACKEND_DIR):
        os.mkdir(MCSNIPERPY_BACKEND_DIR)

    with open(MCSNIPERPY_CONFIG_FILE_PATH, "w") as file:
        config.write(file)

    accounts_path = os.path.join(config["sniper"]["init_path"], "accounts.txt")
    user_config_path = os.path.join(
        config["sniper"]["init_path"], "config.ini")

    if os.path.isfile(accounts_path):
        if log.yes_or_no("Overwrite current accounts file:"):
            with open(
                os.path.join(config["sniper"]["init_path"],
                             "accounts.txt"), "w"
            ) as file:
                file.write(DEFAULT_ACCOUNTS_FILE)
    else:
        with open(
            os.path.join(config["sniper"]["init_path"], "accounts.txt"), "w"
        ) as file:
            file.write(DEFAULT_ACCOUNTS_FILE)

    if os.path.isfile(user_config_path):
        if log.yes_or_no("Overwrite current config file:"):
            with open(
                os.path.join(config["sniper"]["init_path"], "config.ini"), "w"
            ) as file:
                create_user_config().write(file)
    else:
        with open(
            os.path.join(config["sniper"]["init_path"], "config.ini"), "w"
        ) as file:
            create_user_config().write(file)

    log.info("successfully initialized sniper")
